!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                      !
!                Heat exchange sub-model of ELCIRC / SELFE             !
!                     Version 9 (February 23, 2007)                    !
!                                                                      !
!           Center for Coastal and Land-Margin Research                !
!       Department of Environmental Science and Engineering            !
!             OGI School of Science and Engineering,                   !
!               Oregon Health & Science University                     !
!                 Beaverton, Oregon 97006, USA                         !
!                                                                      !
!             Scientific direction: Antonio Baptista                   !
!             Code development: Mike A. Zulauf                         !
!                                                                      !
!         Copyright 1999-2007 Oregon Health and Science University     !
!                        All Rights Reserved                           !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!-----------------------------------------------------------------------
!
! Note: the following global variables (from module global) are used in
!       this code.  This list does not include variables passed in as
!       arguments. . .
!
!       dbl_kind
!       np
!       uu2
!       vv2
!       tnd
!       snd
!       kfp
!       idry
!       nvrt
!       ivcor
!       xlon
!       ylat
!
!-----------------------------------------------------------------------
!
! This file contains four primary subroutines:
!
!	get_wind         (called from within ELCIRC)
!	get_rad          (called from within surf_fluxes)
!	get_precip_flux  (called from within surf_fluxes - IF ENABLED)
!	surf_fluxes      (called from within ELCIRC)
!
! In addition, there are a number of secondary routines and functions
! that are called by those listed above.
!
! The first three of the primary subroutines (get_wind, get_rad, and
! get_precip_flux) are used to specify the atmospheric, radiative, and
! precipitation forcing (though precipitation and evaporation fluxes
! are not always enabled).
!
! The fourth (surf_fluxes) calculates the various components of the
! surface fluxes of heat, momentum, and fresh water (when enabled).
!
! Two versions of get_wind, get_rad, and get_precip_flux are provided
! here.  The first set (in the block in which USE_NETCDF is not defined)
! is supplied here primarily as guidance for users to write their own
! versions. One could specify time- and space-varying values, or
! constant values, etc.
!
! The second set (with the USE_NETCDF preprocessing conditional
! defined) uses the ELCIRC/SELFE netCDF I/O component to specify the
! time- and space-varying forcing quantities.
!
! The precipitation and evaporation fluxes of fresh water through the
! surface (and the get_precip_flux subroutine) are enabled/disabled
! by use of the PREC_EVAP preprocessing conditional.
!
! The code which actually calculates the surface fluxes of heat and
! momentum (surf_fluxes) should typically not need to be modified,
! assuming data is input into the model correctly using get_wind and
! get_rad.
!
! The calling conventions for get_wind and get_rad are listed below:
!---------------------------------------------------------------
!     call get_wind (time, u_air, v_air, p_air, t_air, q_air)
!--------------------
!     scalar
!       time:       model time in seconds
!
!     vectors defined at nodes
!       u_air: east/west component of near-surface wind at a
!              height of 10m, m/s             (map projection)
!       v_air: north/south component of near-surface wind at a
!              height of 10m, m/s             (map projection)
!       p_air: surface atmospheric pressure, Pa
!       t_air: near-surface air temperature at a height of 2m,
!              deg C
!       q_air: near-surface specific humidity at a height of 2m,
!              kg/kg
!---------------------------------------------------------------
!     call get_rad (time, shortwave_d, longwave_d)
!--------------------
!     scalar
!       time:       model time in seconds
!
!     vectors defined at nodes
!       shortwave_d: downwelling shortwave (solar) radiation at
!                    the surface, W/m^2
!                    NOTE: the downwelling shortwave should be
!                          reduced to account for surface albedo
!       longwave_d:  downwelling longwave (IR) radiation at the
!                    surface, W/m^2
!-----------------------------------------------------------------------
!     call get_precip_flux (time, precip_flux)
!--------------------
!     scalar
!       time:       model time in seconds
!
!     vectors defined at nodes
!       precip_flux: downwards precipitation flux at the surface,
!                    kg/m^2/s
!-----------------------------------------------------------------------
!
! Winds in the model need to be aligned with the map projection used
! for the particular grid.  Typically, winds are stored aligned with
! geographic coordinates.
!
! After being input in get_wind, u_air and v_air typically should be
! rotated to the map projection using the rotate_winds subroutine.
!
!-----------------------------------------------------------------------
!
! The standard get_wind, get_rad, and get_precip_flux subroutines (and
! supporting code) depend upon the use of netCDF libraries (Version 3.x)
! and netCDF files written using a specific (though fairly standard)
! format.
!
! get_wind will read from "air" files, which should be formatted along
! these lines (displayed using the ncdump utility):
!
! netcdf sflux_air_1.001 {
! dimensions:
!         nx_grid = 349 ;
!         ny_grid = 277 ;
!         time = UNLIMITED ; // (8 currently)
! variables:
!         float time(time) ;
!                 time:long_name = "Time" ;
!                 time:standard_name = "time" ;
!                 time:units = "days since 2001-01-01" ;
!                 time:base_date = 2001, 1, 1, 0 ;
!         float lon(ny_grid, nx_grid) ;
!                 lon:long_name = "Longitude" ;
!                 lon:standard_name = "longitude" ;
!                 lon:units = "degrees_east" ;
!         float lat(ny_grid, nx_grid) ;
!                 lat:long_name = "Latitude" ;
!                 lat:standard_name = "latitude" ;
!                 lat:units = "degrees_north" ;
!         float uwind(time, ny_grid, nx_grid) ;
!                 uwind:long_name = "Surface Eastward Air Velocity (10m AGL)" ;
!                 uwind:standard_name = "eastward_wind" ;
!                 uwind:units = "m/s" ;
!         float vwind(time, ny_grid, nx_grid) ;
!                 vwind:long_name = "Surface Northward Air Velocity (10m AGL)" ;
!                 vwind:standard_name = "northward_wind" ;
!                 vwind:units = "m/s" ;
!         float prmsl(time, ny_grid, nx_grid) ;
!                 prmsl:long_name = "Pressure reduced to MSL" ;
!                 prmsl:standard_name = "air_pressure_at_sea_level" ;
!                 prmsl:units = "Pa" ;
!         float stmp(time, ny_grid, nx_grid) ;
!                 stmp:long_name = "Surface Air Temperature (2m AGL)" ;
!                 stmp:standard_name = "air_temperature" ;
!                 stmp:units = "K" ;
!         float spfh(time, ny_grid, nx_grid) ;
!                 spfh:long_name = "Surface Specific Humidity (2m AGL)" ;
!                 spfh:standard_name = "specific_humidity" ;
!                 spfh:units = "1" ;
! 
! // global attributes:
!                 :Conventions = "CF-1.0" ;
! }
!
! Note: the names and values of the grid dimensions are arbitrary.
!       The base_date must correspond to the date that corresponds to
!       time = zero for the data.  Variable names are required to be
!       as shown.
!
!       Other metadata is optional - but shown here to document the
!       required names, units, etc.
!
! Likewise, get_rad will read from "rad" files, which should have a
! format similar to this:
!
! netcdf sflux_rad_1.001 {
! dimensions:
!         nx_grid = 349 ;
!         ny_grid = 277 ;
!         time = UNLIMITED ; // (8 currently)
! variables:
!         float time(time) ;
!                 time:long_name = "Time" ;
!                 time:standard_name = "time" ;
!                 time:units = "days since 2001-01-01" ;
!                 time:base_date = 2001, 1, 1, 0 ;
!         float lon(ny_grid, nx_grid) ;
!                 lon:long_name = "Longitude" ;
!                 lon:standard_name = "longitude" ;
!                 lon:units = "degrees_east" ;
!         float lat(ny_grid, nx_grid) ;
!                 lat:long_name = "Latitude" ;
!                 lat:standard_name = "latitude" ;
!                 lat:units = "degrees_north" ;
!         float dlwrf(time, ny_grid, nx_grid) ;
!                 dlwrf:long_name = "Downward Long Wave Radiation Flux" ;
!                 dlwrf:standard_name = "surface_downwelling_longwave_flux_in_air" ;
!                 dlwrf:units = "W/m^2" ;
!         float dswrf(time, ny_grid, nx_grid) ;
!                 dswrf:long_name = "Downward Short Wave Radiation Flux" ;
!                 dswrf:standard_name = "surface_downwelling_shortwave_flux_in_air" ;
!                 dswrf:units = "W/m^2" ;
! 
! // global attributes:
!                 :Conventions = "CF-1.0" ;
! }
!
! Finally, the "prc" files should have a format such as this:
!
! netcdf sflux_prc_1.001 {
! dimensions:
!         nx_grid = 349 ;
!         ny_grid = 277 ;
!         time = UNLIMITED ; // (8 currently)
! variables:
!         float time(time) ;
!                 time:long_name = "Time" ;
!                 time:standard_name = "time" ;
!                 time:units = "days since 2001-01-01" ;
!                 time:base_date = 2001, 1, 1, 0 ;
!         float lon(ny_grid, nx_grid) ;
!                 lon:long_name = "Longitude" ;
!                 lon:standard_name = "longitude" ;
!                 lon:units = "degrees_east" ;
!         float lat(ny_grid, nx_grid) ;
!                 lat:long_name = "Latitude" ;
!                 lat:standard_name = "latitude" ;
!                 lat:units = "degrees_north" ;
!         float prate(time, ny_grid, nx_grid) ;
!                 prate:long_name = "Surface Precipitation Rate" ;
!                 prate:standard_name = "precipitation_flux" ;
!                 prate:units = "kg/m^2/s" ;
! 
! // global attributes:
!                 :Conventions = "CF-1.0" ;
! }
!
!-----------------------------------------------------------------------
!
! Note that net downward heat flux (except for solar) is given by:
!
! net_sfc_flux_d = - (sen_flux + lat_flux + (longwave_u - longwave_d))
!
! The shortwave (solar) flux is penetrative, and is handled separately.
!
      subroutine surf_fluxes (time, u_air, v_air, p_air, &
     &                   t_air, q_air, shortwave_d, &
     &                   sen_flux, lat_flux, longwave_u, longwave_d, &
     &                   tau_xz, tau_yz, &
#ifdef PREC_EVAP
     &                   precip_flux, evap_flux, &
#endif
     &                   nws, fluxsu00, srad00)

        use global, only : dbl_kind, np, uu2, vv2, tnd, snd, &
     &                     kfp, idry, nvrt, ivcor
        implicit none

! input/output variables
        real(kind=dbl_kind), intent(in) :: time, fluxsu00, srad00
        real(kind=dbl_kind), dimension(np), intent(in) :: &
     &    u_air, v_air, p_air, t_air, q_air
        real(kind=dbl_kind), dimension(np), intent(out) :: &
     &    shortwave_d, sen_flux, lat_flux, longwave_u, longwave_d, &
     &    tau_xz, tau_yz
        integer, intent(in) :: nws
#ifdef PREC_EVAP
        real(kind=dbl_kind), dimension(np), intent(out) :: &
     &    precip_flux, evap_flux
#endif
        
! local variables
        integer num_nodes, i_node, sfc_lev
        logical dry
        real(kind=dbl_kind), parameter :: t_freeze = 273.15
        real(kind=dbl_kind), parameter :: stefan = 5.67e-8
        real(kind=dbl_kind), parameter :: emissivity = 1.0
        integer, parameter :: printit = 1000
        character, parameter :: grid_file*50 = 'sflux.gr3'
        real(kind=dbl_kind) :: x_tmp, y_tmp, sflux_frac
        integer i_node_tmp
        logical, save :: first_call = .true.

! define the local variables num_nodes
        num_nodes = np

        open(38,file='fort.38')
        rewind(38)
        write(38,*)
        write(38,*) 'enter surf_fluxes'

! retrieve the downwelling radiative fluxes
        call get_rad (time, shortwave_d, longwave_d)

#ifdef PREC_EVAP
! retrieve the surface precipitation flux
        call get_precip_flux (time, precip_flux)
#endif

! output info to debug file
        write(38,*)
        write(38,*) 'surf_fluxes: time      = ', time
        write(38,*) 'first_call             = ', first_call
        write(38,*) 'num_nodes              = ', num_nodes

! output debugging info
        do i_node = 1, num_nodes

! specify the surface level at this node (depends on coordinate system)
          if (ivcor .eq. -1) then         ! z
            sfc_lev = kfp(i_node)
          else                            ! sigma
            sfc_lev = nvrt
          endif

          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*)
            write(38,*) 'i_node, sfc u, v, T = ', i_node, &
#ifndef SELFE
     &                  uu2(i_node, sfc_lev), &
     &                  vv2(i_node, sfc_lev), &
     &                  tnd(i_node, sfc_lev)
#else /* SELFE */
     &                  uu2(sfc_lev,i_node), &
     &                  vv2(sfc_lev,i_node), &
     &                  tnd(sfc_lev,i_node)
#endif /* SELFE */
            write(38,*) 'u, v, p, T, q (air) = ', u_air(i_node), &
     &                  v_air(i_node), p_air(i_node), t_air(i_node), &
     &                  q_air(i_node)
          endif

        enddo

! calculate the turbulent fluxes at the nodes
        write(38,*) 'above turb_fluxes'
        call turb_fluxes (num_nodes, &
     &                    u_air, v_air, p_air, t_air, q_air, &
     &                    sen_flux, lat_flux, &
#ifdef PREC_EVAP
     &                    evap_flux, &
#endif
     &                    tau_xz, tau_yz)
        write(38,*) 'below turb_fluxes'

! now calculate upwards longwave flux at the surface, using black-body
! equation
        write(38,*) 'calculating longwave_u'
        do i_node = 1, num_nodes

! specify the surface level at this node (depends on coordinate system)
          if (ivcor .eq. -1) then         ! z
            sfc_lev = kfp(i_node)
          else                            ! sigma
            sfc_lev = nvrt
          endif

          longwave_u(i_node) = emissivity * stefan * &
#ifndef SELFE
     &                      ( t_freeze + tnd(i_node, sfc_lev) ) ** 4
#else /* SELFE */
     &                      ( t_freeze + tnd(sfc_lev,i_node) ) ** 4
#endif /* SELFE */

        enddo

! reset flux values if the nws flag is set
        if (nws .eq. 3) then

          open (unit=96, file=grid_file, status='old')

          read(96,*)
          read(96,*) !ne,np
          do i_node = 1, num_nodes
            read(96,*) i_node_tmp, x_tmp, y_tmp, sflux_frac
            sen_flux(i_node)    = sflux_frac * fluxsu00
            shortwave_d(i_node) = sflux_frac * srad00
            lat_flux(i_node) = 0.0
            longwave_u(i_node) = 0.0
            longwave_d(i_node) = 0.0
#ifdef PREC_EVAP
            precip_flux(i_node) = 0.0
            evap_flux(i_node) = 0.0
#endif
          enddo

          close (unit=96)

        endif

        do i_node = 1, num_nodes
          if (mod(i_node-1,printit) .eq. 0) then

! define whether this node is dry or not (depends on coordinate system)
            dry = &
     &          ( (ivcor .eq. -1) .and. (kfp(i_node)  .eq. 0) ) & ! z
     &        .or. &
     &          ( (ivcor .ne. -1) .and. (idry(i_node) .eq. 1) )   !sigma

            if (.not. dry) then
              write(38,*)
              write(38,*) 'i_node = ', i_node
              write(38,*) 'net_sfc_flux_d = ', &
     &                     - sen_flux(i_node) - lat_flux(i_node) &
     &                     - ( longwave_u(i_node) - longwave_d(i_node) )
              write(38,*) 'shortwave_d = ', shortwave_d(i_node)
              write(38,*) 'longwave_d, longwave_u = ', &
     &                     longwave_d(i_node), longwave_u(i_node)
              write(38,*) 'sen_flux, lat_flux = ', &
     &                     sen_flux(i_node), lat_flux(i_node)
#ifdef PREC_EVAP
              write(38,*) 'precip_flux, evap_flux = ', &
     &                     precip_flux(i_node), evap_flux(i_node)
#endif
            else
              write(38,*)
              write(38,*) 'i_node = ', i_node
              write(38,*) 'dry!'
            endif
          endif
        enddo

        close(38)

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

      return
      end
!-----------------------------------------------------------------------
!
! Calculate bulk aerodynamic surface fluxes over water using method of
! Zeng et al., J Clim, v11, p 2628, Oct 1998.
!
! Note: this subroutine uses actual temperatures instead of potential
!       temperatures.  Since the temperature height is only 2m, the
!       difference should be negligible. . .
!
      subroutine turb_fluxes (num_nodes, &
     &                        u_air, v_air, p_air, t_air, q_air, &
     &                        sen_flux, lat_flux, &
#ifdef PREC_EVAP
     &                        evap_flux, &
#endif
     &                        tau_xz, tau_yz)

        use global, only : dbl_kind, uu2, vv2, tnd, snd, &
     &                     kfp, idry, nvrt, ivcor
        implicit none

! input/output variables
        integer, intent(in) :: num_nodes
        real(kind=dbl_kind), dimension(num_nodes), intent(in) :: &
     &    u_air, v_air, p_air, t_air, q_air
        real(kind=dbl_kind), dimension(num_nodes), intent(out) :: &
     &    sen_flux, lat_flux, tau_xz, tau_yz
#ifdef PREC_EVAP
        real(kind=dbl_kind), dimension(num_nodes), intent(out) :: &
     &    evap_flux
#endif

! local variables
        integer i_node, iter, sfc_lev
        integer, parameter :: max_iter = 10
        real(kind=dbl_kind) u_star, theta_star, q_star, z_0, monin
        real(kind=dbl_kind) zeta_u, zeta_t, one_third, w_star, mix_ratio
        real(kind=dbl_kind) t_v, speed, psi_m, psi_h
        real(kind=dbl_kind) re, z_0_t, e_sfc, q_sfc, rho_air, rb
        real(kind=dbl_kind) theta_air, theta_v_air, delta_theta, delta_q
        real(kind=dbl_kind) delta_theta_v, theta_v_star, speed_res, tau
        real(kind=dbl_kind) speed_air, speed_water, esat_flat_r
        real(kind=dbl_kind), parameter :: speed_air_warn = 50.0
        real(kind=dbl_kind), parameter :: speed_air_stop = 100.0
        real(kind=dbl_kind), parameter :: speed_water_warn = 5.0
        real(kind=dbl_kind), parameter :: speed_water_stop = 20.0
        real(kind=dbl_kind), parameter :: z_t = 2.0
        real(kind=dbl_kind), parameter :: z_u = 10.0
        real(kind=dbl_kind), parameter :: a1 = 0.013
        real(kind=dbl_kind), parameter :: a2 = 0.11
        real(kind=dbl_kind), parameter :: b1 = 2.67
        real(kind=dbl_kind), parameter :: b2 = -2.57
        real(kind=dbl_kind), parameter :: nu = 1.46e-5
        real(kind=dbl_kind), parameter :: beta = 1.0
        real(kind=dbl_kind), parameter :: g = 9.81
        real(kind=dbl_kind), parameter :: z_i = 1000.0
        real(kind=dbl_kind), parameter :: karman = 0.4
        real(kind=dbl_kind), parameter :: zeta_m = -1.574
        real(kind=dbl_kind), parameter :: zeta_h = -0.465
        real(kind=dbl_kind), parameter :: t_freeze = 273.15
        real(kind=dbl_kind), parameter :: epsilon_r = 0.6220
        real(kind=dbl_kind), parameter :: c_p_air = 1004.0
        real(kind=dbl_kind), parameter :: latent = 2.501e6
        real(kind=dbl_kind), parameter :: r_air = 287.0
        integer, parameter :: printit = 1000
        logical converged, dry

        write(38,*) 'enter turb_fluxes'

! precalculate constants
        one_third = 1.0 / 3.0

! now loop over all points
        do i_node = 1, num_nodes
          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*)
            write(38,*) 'i_node = ', i_node
          endif

! define whether this node is dry or not (depends on coordinate system)
          dry = &
     &        ( (ivcor .eq. -1) .and. (kfp(i_node)  .eq. 0) ) & ! z
     &      .or. &
     &        ( (ivcor .ne. -1) .and. (idry(i_node) .eq. 1) )   !sigma

! if this point isn't dry, then calculate fluxes (if dry, then skip)
        if (.not. dry) then

! specify the surface level at this node (depends on coordinate system)
          if (ivcor .eq. -1) then         ! z
            sfc_lev = kfp(i_node)
          else                            ! sigma
            sfc_lev = nvrt
          endif

! calculate q_sfc from e_sfc
! (e_sfc reduced for salinity using eqn from Smithsonian Met Tables)
#ifndef SELFE
          e_sfc = (1.0 - 0.000537 * snd(i_node, sfc_lev)) &
     &          * esat_flat_r(tnd(i_node, sfc_lev) + t_freeze)
#else /* SELFE */
          e_sfc = (1.0 - 0.000537 * snd(sfc_lev,i_node)) &
     &          * esat_flat_r(tnd(sfc_lev,i_node) + t_freeze)
#endif /* SELFE */
          q_sfc = epsilon_r * e_sfc &
     &          / ( p_air(i_node) - e_sfc * (1.0 - epsilon_r) )

! calculate the water vapor mixing ratio of the air
          mix_ratio = q_air(i_node) / (1.0 - q_air(i_node))

! calculate theta_air, theta_v_air, delta_theta, delta_q,
! and delta_theta_v
          theta_air = (t_air(i_node) + t_freeze) + 0.0098*z_t
          theta_v_air = theta_air * (1.0 + 0.608 * mix_ratio)
          delta_theta = theta_air - &
#ifndef SELFE
     &                  (tnd(i_node, sfc_lev) + t_freeze)
#else /* SELFE */
     &                  (tnd(sfc_lev,i_node) + t_freeze)
#endif /* SELFE */
          delta_q = q_air(i_node) - q_sfc
          delta_theta_v = delta_theta * (1.0 + 0.608 * mix_ratio) &
     &                  + 0.608 * theta_air * delta_q

! calculate the air virtual temperature and density
          t_v = (t_air(i_node) + t_freeze) * (1.0 + 0.608 * mix_ratio)
          rho_air = p_air(i_node) / (r_air * t_v)
          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*) 'e_sfc, q_sfc, mix_ratio = ', &
     &                   e_sfc, q_sfc, mix_ratio
            write(38,*) 'theta_air, theta_v_air, delta_theta = ', &
     &                   theta_air, theta_v_air, delta_theta
            write(38,*) 'delta_q, delta_theta_v, t_v = ', &
     &                   delta_q, delta_theta_v, t_v
            write(38,*) 'rho_air = ', rho_air
          endif

! check air and water speeds, and give warnings and/or bombs for
! excessive values
          speed_air = sqrt( u_air(i_node)*u_air(i_node) + &
     &                      v_air(i_node)*v_air(i_node) )
          speed_water &
#ifndef SELFE
     &      = sqrt( uu2(i_node, sfc_lev)*uu2(i_node, sfc_lev) + &
     &              vv2(i_node, sfc_lev)*vv2(i_node, sfc_lev) )
#else /* SELFE */
     &      = sqrt( uu2(sfc_lev,i_node)*uu2(sfc_lev,i_node) + &
     &              vv2(sfc_lev,i_node)*vv2(sfc_lev,i_node) )
#endif /* SELFE */

          if (speed_air .gt. speed_air_stop) then
            write(11,*)
            write(11,*) 'speed_air exceeds ', speed_air_stop
            stop
          else if (speed_air .gt. speed_air_warn) then
            write(12,*)
            write(12,*) 'speed_air exceeds ', speed_air_warn
          endif

          if (speed_water .gt. speed_water_stop) then
            write(11,*)
            write(11,*) 'speed_water exceeds ', speed_water_stop,i_node,uu2(sfc_lev,i_node),vv2(sfc_lev,i_node)
            stop
          else if (speed_water .gt. speed_water_warn) then
            write(12,*)
            write(12,*) 'speed_water exceeds ', speed_water_warn,i_node
          endif

! begin with initial values of u_star, w_star, and speed
          u_star = 0.06
          w_star = 0.5
          if (delta_theta_v .ge. 0) then                    ! stable
            speed = &
     &        max( sqrt( &
#ifndef SELFE
     &               (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &               (v_air(i_node) - vv2(i_node, sfc_lev))**2 ), &
#else /* SELFE */
     &               (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &               (v_air(i_node) - vv2(sfc_lev,i_node))**2 ), &
#endif /* SELFE */
     &             0.1d0)
          else                                              ! unstable
            speed = &
#ifndef SELFE
     &        sqrt( (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &              (v_air(i_node) - vv2(i_node, sfc_lev))**2 + &
#else /* SELFE */
     &        sqrt( (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &              (v_air(i_node) - vv2(sfc_lev,i_node))**2 + &
#endif /* SELFE */
     &              (beta * w_star)**2 )
          endif

! now loop to obtain good initial values for u_star and z_0
          do iter = 1, 5
            z_0 = a1 * u_star * u_star / g + a2 * nu / u_star
            u_star = karman * speed / log(z_u/z_0)
          enddo

! calculate rb (some stability parameter from Xubin's code?)
          rb = g * z_u * delta_theta_v / (theta_v_air * speed * speed)

! calculate initial values for zeta_u, monin, zeta_t
          if (rb .ge. 0) then                      ! neutral or stable
            zeta_u = rb * log(z_u/z_0) &
     &             / (1.0 - 0.5*min(rb,0.19d0))
          else
            zeta_u = rb * log(z_u/z_0)
          endif
          monin = z_u / zeta_u
          zeta_t = z_t / monin
          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*) 'speed, z_0, u_star = ', &
     &                   speed, z_0, u_star
            write(38,*) 'zeta_u, zeta_t, monin = ', &
     &                   zeta_u, zeta_t, monin
          endif

! iterate a maximum of max_iter times
          iter = 0
          converged = .false.
100       continue
            iter = iter + 1

! Calculate the roughness lengths
            z_0 = a1 * u_star * u_star / g + a2 * nu / u_star
            re = u_star * z_0 / nu
            z_0_t = z_0 / exp(b1 * (re**0.25) + b2)
            if (mod(i_node-1,printit) .eq. 0) then
              write(38,*) 're, z_0, z_0_t = ', &
     &                     re, z_0, z_0_t
            endif

! calculate the zetas
            zeta_u = z_u / monin
            zeta_t = z_t / monin

! apply asymptotic limit to stable conditions
            if (zeta_t .gt. 2.5) then
              converged = .true.
              zeta_t = 2.5
              monin = z_t / zeta_t
              zeta_u = z_u / monin
              if (mod(i_node-1,printit) .eq. 0) then
                write(38,*) 'limiting zeta_u, zeta_t, monin!'
              endif
            endif

! caulculate u_star, depending on zeta
            if (zeta_u .lt. zeta_m) then                ! very unstable

              u_star = speed * karman &
     &               / ( log(zeta_m*monin/z_0) &
     &                   - psi_m(zeta_m) &
! extra term?
     &                   + psi_m(z_0/monin) &
     &                   + 1.14 * ((-zeta_u)**(one_third) - &
     &                           (-zeta_m)**(one_third)) )

            else if (zeta_u .lt. 0.0) then              ! unstable

              u_star = speed * karman &
     &               / ( log(z_u/z_0) &
     &                   - psi_m(zeta_u) &
! extra term?
     &                   + psi_m(z_0/monin) &
     &                 )

            else if (zeta_u .le. 1.0) then              ! neutral/stable

              u_star = speed * karman &
     &               / ( log(z_u/z_0) + 5.0*zeta_u &
! extra term?
     &                   - 5.0*z_0/monin &
     &                 )

            else                                        ! very stable

              u_star = speed * karman &
     &               / ( log(monin/z_0) + 5.0 &
     &                   + 5.0*log(zeta_u) &
! extra term?
     &                   - 5.0*z_0/monin &
     &                   + zeta_u - 1.0 )

            endif

! caulculate theta_star and q_star, depending on zeta
            if (zeta_t .lt. zeta_h) then                ! very unstable

              theta_star = karman * delta_theta &
     &                   / ( log(zeta_h*monin/z_0_t) &
     &                       - psi_h(zeta_h) &
! extra term?
     &                       + psi_h(z_0_t/monin) &
     &                       + 0.8 * ((-zeta_h)**(-one_third) - &
     &                                (-zeta_t)**(-one_third)) )

              q_star = karman * delta_q &
     &                   / ( log(zeta_h*monin/z_0_t) &
     &                       - psi_h(zeta_h) &
! extra term?
     &                       + psi_h(z_0_t/monin) &
     &                       + 0.8 * ((-zeta_h)**(-one_third) - &
     &                                (-zeta_t)**(-one_third)) )

            else if (zeta_t .lt. 0.0) then              ! unstable

              theta_star = karman * delta_theta &
     &                   / ( log(z_t/z_0_t) &
     &                       - psi_h(zeta_t) &
! extra term?
     &                       + psi_h(z_0_t/monin) &
     &                     )

              q_star = karman * delta_q &
     &                   / ( log(z_t/z_0_t) &
     &                       - psi_h(zeta_t) &
! extra term?
     &                       + psi_h(z_0_t/monin) &
     &                     )

            else if (zeta_t .lt. 1.0) then              ! neutral/stable

              theta_star = karman * delta_theta &
     &                   / ( log(z_t/z_0_t) &
     &                       + 5.0*zeta_t &
! extra term?
     &                       - 5.0*z_0_t/monin &
     &                     )

              q_star = karman * delta_q &
     &                   / ( log(z_t/z_0_t) &
     &                       + 5.0*zeta_t &
! extra term?
     &                       - 5.0*z_0_t/monin &
     &                     )

            else                                        ! very stable

              theta_star = karman * delta_theta &
     &                   / ( log(monin/z_0_t) + 5.0 &
     &                       + 5.0*log(zeta_t) &
! extra term?
     &                       - 5.0*z_0_t/monin &
     &                       + zeta_t - 1.0 )

              q_star = karman * delta_q &
     &                   / ( log(monin/z_0_t) + 5.0 &
     &                       + 5.0*log(zeta_t) &
! extra term?
     &                       - 5.0*z_0_t/monin &
     &                       + zeta_t - 1.0 )

            endif

! calculate theta_v_star and monin
            theta_v_star = theta_star * (1.0 + 0.608 * mix_ratio) &
     &                   + 0.608 * theta_air * q_star
            monin = theta_v_air * u_star * u_star &
     &            / (karman * g * theta_v_star)

! depending on surface layer stability, calculate the effective
! near-surface wind speed
! (ie relative to the flowing water surface)
            if (delta_theta_v .ge. 0.0) then                  ! stable

              speed = &
     &          max( sqrt( &
#ifndef SELFE
     &                 (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &                 (v_air(i_node) - vv2(i_node, sfc_lev))**2 ), &
#else /* SELFE */
     &                 (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &                 (v_air(i_node) - vv2(sfc_lev,i_node))**2 ), &
#endif /* SELFE */
     &               0.1d0)

            else                                              ! unstable

! calculate the convective velocity scale
              w_star = (-g * theta_v_star * u_star * z_i / theta_v_air) &
     &                 ** one_third

              speed = &
#ifndef SELFE
     &          sqrt( (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &                (v_air(i_node) - vv2(i_node, sfc_lev))**2 + &
#else /* SELFE */
     &          sqrt( (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &                (v_air(i_node) - vv2(sfc_lev,i_node))**2 + &
#endif /* SELFE */
     &                (beta * w_star)**2 )

            endif
            if (mod(i_node-1,printit) .eq. 0) then
              write(38,*) 'iter, u_star, q_star, theta_star = ', &
     &                     iter, u_star, q_star, theta_star
              write(38,*) 'iter, theta_v_star, monin, speed = ', &
     &                     iter, theta_v_star, monin, speed
              write(38,*) 'iter, zeta_u, zeta_t = ', &
     &                     iter, zeta_u, zeta_t
            endif

! bottom of main iteration loop
          if (.not. converged .and. iter .lt. max_iter) goto 100

! calculate fluxes
          sen_flux(i_node) = - rho_air * c_p_air * u_star * theta_star
          lat_flux(i_node) = - rho_air * latent * u_star * q_star
#ifdef PREC_EVAP
          evap_flux(i_node) = - rho_air * u_star * q_star
#endif

! calculate wind stresses
          speed_res = &
#ifndef SELFE
     &          sqrt( (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &                (v_air(i_node) - vv2(i_node, sfc_lev))**2 )
#else /* SELFE */
     &          sqrt( (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &                (v_air(i_node) - vv2(sfc_lev,i_node))**2 )
#endif /* SELFE */
          if (speed_res .gt. 0.0) then
            tau = rho_air * u_star * u_star * speed_res / speed
            tau_xz(i_node) = - tau &
#ifndef SELFE
     &                     * (u_air(i_node) - uu2(i_node, sfc_lev)) &
#else /* SELFE */
     &                     * (u_air(i_node) - uu2(sfc_lev,i_node)) &
#endif /* SELFE */
     &                     / speed_res
            tau_yz(i_node) = - tau &
#ifndef SELFE
     &                     * (v_air(i_node) - vv2(i_node, sfc_lev)) &
#else /* SELFE */
     &                     * (v_air(i_node) - vv2(sfc_lev,i_node)) &
#endif /* SELFE */
     &                     / speed_res
          else
            tau_xz(i_node) = 0.0
            tau_yz(i_node) = 0.0
          endif

          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*) 'sen_flux, lat_flux = ', &
     &                   sen_flux(i_node), lat_flux(i_node)
            write(38,*) 'tau_xz, tau_yz = ', &
     &                   tau_xz(i_node), tau_yz(i_node)
          endif

! end of wet/dry block
        endif

! end of loop over points
        enddo

        write(38,*) 'exit turb_fluxes'

      return
      end
!-----------------------------------------------------------------------
!
! Calculate saturation vapor pressure using the eighth order relative
! error norm method of Flatau et al., J Applied Meteo, v31, p 1507,
! Dec 1992.
!
      function esat_flat_r(t)
        use global, only : dbl_kind
        implicit none
        real(dbl_kind)             :: esat_flat_r
        real(dbl_kind), intent(in) :: t
        real(dbl_kind)             :: t_eff
        real(dbl_kind), parameter :: &
     &        c0= 6.11583699d+02,  c1= 0.444606896d+02, &
     &        c2= 0.143177157d+01, c3= 0.264224321d-01, &
     &        c4= 0.299291081d-03, c5= 0.203154182d-05, &
     &        c6= 0.702620698d-08, c7= 0.379534310d-11, &
     &        c8=-0.321582393d-13

! t     : temperature in K
! t_eff : effective temperature in C

        t_eff = max(-85.0_dbl_kind,t-273.16_dbl_kind)

        esat_flat_r = c0+t_eff*(c1+t_eff*(c2+t_eff*(c3+t_eff*(c4+t_eff*&
     &                         (c5+t_eff*(c6+t_eff*(c7+t_eff*c8)))))))

      return
      end
!-----------------------------------------------------------------------
      function psi_m(zeta)
        use global, only : dbl_kind
        implicit none
        real(dbl_kind)             :: psi_m
        real(dbl_kind), intent(in) :: zeta
        real(dbl_kind) :: chi, half_pi

        half_pi = 2.0 * atan(1.0)
        chi = (1.0 - 16.0 * zeta)**0.25
        psi_m = 2.0 * log( 0.5 * (1.0 + chi) ) + &
     &          log( 0.5 * (1.0 + chi*chi) ) - &
     &          2.0 * atan(chi) + half_pi

      return
      end
!-----------------------------------------------------------------------
      function psi_h(zeta)
        use global, only : dbl_kind
        implicit none
        real(dbl_kind)             :: psi_h
        real(dbl_kind), intent(in) :: zeta
        real(dbl_kind) :: chi

        chi = (1.0 - 16.0 * zeta)**0.25
        psi_h = 2.0 * log( 0.5 * (1.0 + chi*chi) )

      return
      end
!-----------------------------------------------------------------------
      subroutine get_albedo (albedo, num_nodes_out)
        use global, only : dbl_kind
        implicit none
        integer, intent(in) :: num_nodes_out
        real(kind=dbl_kind), intent(out), dimension(num_nodes_out) :: &
     &    albedo

        albedo = 0.06

      return
      end
!-----------------------------------------------------------------------
      subroutine rotate_winds (u, v, num_nodes_out)

        use global, only : dbl_kind
        implicit none

! input/output variables
        integer num_nodes_out
        real(kind=dbl_kind) u(num_nodes_out), v(num_nodes_out)

! local variables
        integer i_node, i_node_tmp, alloc_stat
        real(kind=dbl_kind) x_tmp, y_tmp, speed, dir
        real(kind=dbl_kind) pi, deg_to_rad
        real(kind=dbl_kind), save, allocatable, dimension(:) :: &
     &    rotate_angle
        character, parameter :: rot_file*50 = 'windrot_geo2proj.gr3'
        logical, save :: first_call = .true.

        pi = 4.0 * atan(1.0d0)
        deg_to_rad = pi / 180.0d0

! if this is the first call to this subroutine, then read in the angles
! that the winds will need to be rotated by
! (convert degrees to radians)
        if (first_call) then

! allocate array for needed size
          allocate (rotate_angle(num_nodes_out), stat=alloc_stat)
          call check_allocation('rotate_angle', 'rotate_winds', &
     &                          alloc_stat)

          open(unit=13, file=rot_file, status='old')
          read(13,*) ! header
          read(13,*) ! ne, np

          do i_node =1, num_nodes_out
            read(13,*) i_node_tmp, x_tmp, y_tmp, rotate_angle(i_node)
            rotate_angle(i_node) = rotate_angle(i_node) * deg_to_rad
          enddo

          close(unit=13)

        endif

! rotate winds
        do i_node =1, num_nodes_out

! calculate speed and direction (geographic)
          dir = atan2(-u(i_node),-v(i_node))
          speed = sqrt(u(i_node)*u(i_node) + v(i_node)*v(i_node))

! add rotation angle
          dir = dir + rotate_angle(i_node)

! calculate new u and v components
          u(i_node) = -speed * sin(dir)
          v(i_node) = -speed * cos(dir)

        enddo

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

      return
      end
!-----------------------------------------------------------------------
      subroutine check_allocation(variable, location, status)
        implicit none
        character(*), intent(in) :: variable, location
        integer, intent(in) :: status

        if (status .ne. 0) then
          write(11,*)
          write(11,*) 'allocation error in: ', location
          write(11,*) 'for: ', variable
          stop
        endif

      end subroutine check_allocation
!-----------------------------------------------------------------------
#ifndef USE_NETCDF /* USE_NETCDF not defined */
!-----------------------------------------------------------------------
      subroutine get_wind (time, u_air_node, v_air_node, p_air_node, &
     &                     t_air_node, q_air_node)

        use global, only : dbl_kind, np
        implicit none
        
        real(kind=dbl_kind), intent(in) :: time
        real(kind=dbl_kind), dimension(np), intent(out) :: &
     &    u_air_node, v_air_node, p_air_node, t_air_node, q_air_node
        
! local variables
        integer num_nodes_out, i_node
        logical, save :: first_call = .true.
        real(kind=dbl_kind), parameter :: t_freeze = 273.15

! define the local variables num_nodes_out
        num_nodes_out = np

        open(39,file='fort.39')
        rewind(39)

! output info to debug file
        write(39,*)
        write(39,*) 'get_wind: time         = ', time
        write(39,*) 'first_call             = ', first_call
        write(39,*) 'num_nodes_out          = ', num_nodes_out

! for the first call only, initialize starting date, datasets, etc
! (as needed)
        if (first_call) then
        endif

! specify atmospheric state using whatever logic is desired
        write(39,*) 'specifying surface properties'
        do i_node = 1, num_nodes_out
          u_air_node(i_node) = 0.0d0
          v_air_node(i_node) = 0.0d0
          t_air_node(i_node) = 280.0d0
          q_air_node(i_node) = 8.0d-3
          p_air_node(i_node) = 1.01d5
        enddo

! convert air temperatures to Celcius
        t_air_node = t_air_node - t_freeze

! rotate the winds from geographic to the grid's map projection
        call rotate_winds (u_air_node, v_air_node, num_nodes_out)

        close(39)

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

      return
      end
!-----------------------------------------------------------------------
      subroutine get_rad (time, shortwave_d, longwave_d)

        use global, only : dbl_kind, np
        implicit none
        
        real(kind=dbl_kind), intent(in) :: time
        real(kind=dbl_kind), dimension(np), intent(out) :: &
     &    longwave_d, shortwave_d
        
! local variables
        integer num_nodes_out, i_node
        logical, save :: first_call = .true.
        real(kind=dbl_kind), dimension(np) :: albedo

! define the local variables num_nodes_out
        num_nodes_out = np

        open(40,file='fort.40')
        rewind(40)

! output info to debug file
        write(40,*)
        write(40,*) 'get_rad:  time         = ', time
        write(40,*) 'first_call             = ', first_call
        write(40,*) 'num_nodes_out          = ', num_nodes_out

! for the first call only, initialize starting date, datasets, etc
! (as needed)
        if (first_call) then
        endif

! specify downwelling fluxes using whatever logic is desired
        write(40,*) 'specifying shortwave and longwave'
        do i_node = 1, num_nodes_out
          shortwave_d(i_node) = 250.0d0
          longwave_d(i_node) = 350.0d0
        enddo

! get the albedo
        write(40,*) 'calculating albedo'
        call get_albedo (albedo, num_nodes_out)

! reduce the downwards shortwave flux at the surface by the albedo
! (ensure there are no negative values from interpolation, etc)
        write(40,*) 'reducing shortwave'
        do i_node = 1, num_nodes_out
          shortwave_d(i_node) = &
     &      max( (1.0_dbl_kind - albedo(i_node))*shortwave_d(i_node), &
     &           0.0_dbl_kind )
        enddo

        close(40)

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

      return
      end
!-----------------------------------------------------------------------
      subroutine get_precip_flux (time, precip_flux)

        use global, only : dbl_kind, np
        implicit none
        
        real(kind=dbl_kind), intent(in) :: time
        real(kind=dbl_kind), dimension(np), intent(out) :: precip_flux
        
! local variables
        integer num_nodes_out, i_node
        logical, save :: first_call = .true.

! define the local variables num_nodes_out
        num_nodes_out = np

        open(41,file='fort.41')
        rewind(41)

! output info to debug file
        write(41,*)
        write(41,*) 'get_precip_flux:  time = ', time
        write(41,*) 'first_call             = ', first_call
        write(41,*) 'num_nodes_out          = ', num_nodes_out

! for the first call only, initialize starting date, datasets, etc
! (as needed)
        if (first_call) then
        endif

! specify precipitation flux using whatever logic is desired
        write(41,*) 'specifying precipitation flux'
        do i_node = 1, num_nodes_out
          precip_flux(i_node) = 0.0d0
        enddo

        close(41)

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

      return
      end
!-----------------------------------------------------------------------
#else              /* USE_NETCDF is defined */
!-----------------------------------------------------------------------
      module netcdf_io

        use global, only : dbl_kind
        implicit none
        
        integer, parameter :: max_files = 1000
        integer, parameter :: max_times = 1000

        type dataset_info
          character name*50
          logical :: exist = .false.
          integer :: num_files = 0
          integer :: nx = 0
          integer :: ny = 0
          integer :: num_nodes = 0
          integer :: num_elems = 0
#ifndef NO_TR_15581  /* TR_15581 is implemented */
          real(kind=dbl_kind), allocatable, dimension(:,:) :: lon, lat
          real(kind=dbl_kind), allocatable, dimension(:,:) :: weight
          integer, allocatable, dimension(:) :: node_i, node_j
          integer, allocatable, dimension(:,:) :: node_num, elem_nodes
          integer, allocatable, dimension(:) :: in_elem_for_out_node
#else  /* TR_15581 is NOT implemented */
          real(kind=dbl_kind),     pointer, dimension(:,:) :: lon, lat
          real(kind=dbl_kind),     pointer, dimension(:,:) :: weight
          integer,     pointer, dimension(:) :: node_i, node_j
          integer,     pointer, dimension(:,:) :: node_num, elem_nodes
          integer,     pointer, dimension(:) :: in_elem_for_out_node
#endif  /* NO_TR_15581 block */
          integer :: num_times = 0
          real(kind=dbl_kind), dimension(max_times) :: times
          integer, dimension(max_times) :: file_num_for_time
          integer, dimension(max_times) :: time_num_for_time
          integer, dimension(max_files) :: jdate_for_file
          real(kind=dbl_kind) :: max_window_hours
          real(kind=dbl_kind) :: relative_weight
          logical :: fail_if_missing
        end type dataset_info
        
        integer             :: start_year  = -9999
        integer             :: start_month = -9999
        integer             :: start_day   = -9999
        real(kind=dbl_kind) :: start_hour  = -9999.0
        real(kind=dbl_kind) :: utc_start   = -9999.0
        integer             :: start_jdate
        real(kind=dbl_kind) :: start_frac_jdate = -9999.0

        real(kind=dbl_kind) :: air_1_relative_weight = 1.0
        real(kind=dbl_kind) :: air_2_relative_weight = 2.0
        real(kind=dbl_kind) :: air_1_max_window_hours = 24.0
        real(kind=dbl_kind) :: air_2_max_window_hours = 24.0
        logical             :: air_1_fail_if_missing = .true.
        logical             :: air_2_fail_if_missing = .false.
        character (len=50)  :: air_1_file = 'sflux_air_1'
        character (len=50)  :: air_2_file = 'sflux_air_2'
        character (len=50)  :: uwind_name = 'uwind'
        character (len=50)  :: vwind_name = 'vwind'
        character (len=50)  :: prmsl_name = 'prmsl'
        character (len=50)  :: stmp_name  = 'stmp'
        character (len=50)  :: spfh_name  = 'spfh'

        real(kind=dbl_kind) :: rad_1_relative_weight = 1.0
        real(kind=dbl_kind) :: rad_2_relative_weight = 2.0
        real(kind=dbl_kind) :: rad_1_max_window_hours = 24.0
        real(kind=dbl_kind) :: rad_2_max_window_hours = 24.0
        logical             :: rad_1_fail_if_missing = .true.
        logical             :: rad_2_fail_if_missing = .false.
        character (len=50)  :: rad_1_file = 'sflux_rad_1'
        character (len=50)  :: rad_2_file = 'sflux_rad_2'
        character (len=50)  :: dlwrf_name = 'dlwrf'
        character (len=50)  :: dswrf_name = 'dswrf'
       
        real(kind=dbl_kind) :: prc_1_relative_weight = 1.0
        real(kind=dbl_kind) :: prc_2_relative_weight = 2.0
        real(kind=dbl_kind) :: prc_1_max_window_hours = 24.0
        real(kind=dbl_kind) :: prc_2_max_window_hours = 24.0
        logical             :: prc_1_fail_if_missing = .true.
        logical             :: prc_2_fail_if_missing = .false.
        character (len=50)  :: prc_1_file = 'sflux_prc_1'
        character (len=50)  :: prc_2_file = 'sflux_prc_2'
        character (len=50)  :: prate_name = 'prate'

        namelist /sflux_inputs/ &
     &    start_year, start_month, start_day, start_hour, utc_start, &
     &    air_1_relative_weight, air_2_relative_weight,              &
     &    air_1_max_window_hours, air_2_max_window_hours,            &
     &    air_1_fail_if_missing, air_2_fail_if_missing,              &
     &    air_1_file, air_2_file,                                    &
     &    uwind_name, vwind_name, prmsl_name, stmp_name, spfh_name,  &
     &    rad_1_relative_weight, rad_2_relative_weight,              &
     &    rad_1_max_window_hours, rad_2_max_window_hours,            &
     &    rad_1_fail_if_missing, rad_2_fail_if_missing,              &
     &    rad_1_file, rad_2_file,                                    &
     &    dlwrf_name, dswrf_name,                                    &
     &    prc_1_relative_weight, prc_2_relative_weight,              &
     &    prc_1_max_window_hours, prc_2_max_window_hours,            &
     &    prc_1_fail_if_missing, prc_2_fail_if_missing,              &
     &    prc_1_file, prc_2_file,                                    &
     &    prate_name

      end module netcdf_io
!-----------------------------------------------------------------------
      subroutine get_wind (time, u_air_node, v_air_node, p_air_node, &
     &                     t_air_node, q_air_node)

        use global, only : dbl_kind, np
        use netcdf_io
        implicit none
        
        real(kind=dbl_kind), intent(in) :: time
        real(kind=dbl_kind), dimension(np), intent(out) :: &
     &    u_air_node, v_air_node, p_air_node, t_air_node, q_air_node
        
! local variables
        integer num_nodes_out
        logical, save :: first_call = .true.
        type(dataset_info), save :: dataset_1, dataset_2
        real(kind=dbl_kind) time_now
        real(kind=dbl_kind), parameter :: secs_per_day = 86400.0
        real(kind=dbl_kind), parameter :: t_freeze = 273.15
        character data_name*50

! define the local variables num_nodes_out
        num_nodes_out = np

        open(39,file='fort.39')
        rewind(39)

! output info to debug file
        write(39,*)
        write(39,*) 'get_wind: time         = ', time
        write(39,*) 'first_call             = ', first_call
        write(39,*) 'num_nodes_out          = ', num_nodes_out

! for the first call only, initialize starting date, datasets, etc 
        if (first_call) then
        
          call get_sflux_inputs ()
          
! setup datasets
          dataset_1%name             = air_1_file
          dataset_1%max_window_hours = air_1_max_window_hours
          dataset_1%fail_if_missing  = air_1_fail_if_missing
          dataset_1%relative_weight  = air_1_relative_weight

          dataset_2%name             = air_2_file
          dataset_2%max_window_hours = air_2_max_window_hours
          dataset_2%fail_if_missing  = air_2_fail_if_missing
          dataset_2%relative_weight  = air_2_relative_weight

! output some details
          write(16,*)
          write(16,*) 'get_wind: sflux_inputs'
          write(16,*) '  start_year             = ', start_year
          write(16,*) '  start_month            = ', start_month
          write(16,*) '  start_day              = ', start_day
          write(16,*) '  start_hour             = ', start_hour
          write(16,*) '  utc_start              = ', utc_start
          write(16,*) '  start_frac_jdate       = ', start_frac_jdate
          write(16,*) '  air_1_file             = ', &
     &                   trim(air_1_file)
          write(16,*) '  air_1_max_window_hours = ', &
     &                   air_1_max_window_hours
          write(16,*) '  air_1_fail_if_missing  = ', &
     &                   air_1_fail_if_missing
          write(16,*) '  air_1_relative_weight  = ', &
     &                   air_1_relative_weight
          write(16,*) '  air_2_file             = ', &
     &                   trim(air_2_file)
          write(16,*) '  air_2_max_window_hours = ', &
     &                   air_2_max_window_hours
          write(16,*) '  air_2_fail_if_missing  = ', &
     &                   air_2_fail_if_missing
          write(16,*) '  air_2_relative_weight  = ', &
     &                   air_2_relative_weight
          write(16,*) '  uwind_name             = ', &
     &                   trim(uwind_name)
          write(16,*) '  vwind_name             = ', &
     &                   trim(vwind_name)
          write(16,*) '  prmsl_name             = ', &
     &                   trim(prmsl_name)
          write(16,*) '  stmp_name              = ', &
     &                   trim(stmp_name)
          write(16,*) '  spfh_name              = ', &
     &                   trim(spfh_name)

! get basic info for dataset
          call get_dataset_info (dataset_1)
          call get_dataset_info (dataset_2)

        endif

! get the current time
        time_now = start_frac_jdate + time/secs_per_day
        write(39,*) 'current jdate        = ', time_now

! get the data at this time
        data_name = trim(uwind_name)
        call combine_sflux_data &
     &    (time_now, dataset_1, dataset_2, &
     &     dataset_1%exist, dataset_2%exist, &
     &     data_name, u_air_node, &
     &     num_nodes_out)
        
        data_name = trim(vwind_name)
        call combine_sflux_data &
     &    (time_now, dataset_1, dataset_2, &
     &     dataset_1%exist, dataset_2%exist, &
     &     data_name, v_air_node, &
     &     num_nodes_out)
        
        data_name = trim(prmsl_name)
        call combine_sflux_data &
     &    (time_now, dataset_1, dataset_2, &
#ifndef MM5     /* MM5 not defined  */
     &     dataset_1%exist, dataset_2%exist, &
#else           /* MM5 is defined   */
     &     dataset_1%exist, .false., &
#endif          /* end of MM5 block */
     &     data_name, p_air_node, &
     &     num_nodes_out)
        
        data_name = trim(stmp_name)
        call combine_sflux_data &
     &    (time_now, dataset_1, dataset_2, &
     &     dataset_1%exist, dataset_2%exist, &
     &     data_name, t_air_node, &
     &     num_nodes_out)
        
        data_name = trim(spfh_name)
        call combine_sflux_data &
     &    (time_now, dataset_1, dataset_2, &
     &     dataset_1%exist, dataset_2%exist, &
     &     data_name, q_air_node, &
     &     num_nodes_out)
        
! convert air temperatures to Celcius
        t_air_node = t_air_node - t_freeze

! rotate the winds from geographic to the grid's map projection
        call rotate_winds (u_air_node, v_air_node, num_nodes_out)

        close(39)

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

      return
      end
!-----------------------------------------------------------------------
      subroutine get_rad (time, shortwave_d, longwave_d)

        use global, only : dbl_kind, np
        use netcdf_io
        implicit none
        
        real(kind=dbl_kind), intent(in) :: time
        real(kind=dbl_kind), dimension(np), intent(out) :: &
     &    longwave_d, shortwave_d
        
! local variables
        integer num_nodes_out, i_node
        logical, save :: first_call = .true.
        type(dataset_info), save :: dataset_1, dataset_2
        real(kind=dbl_kind) time_now
        real(kind=dbl_kind), dimension(np) :: albedo
        real(kind=dbl_kind), parameter :: secs_per_day = 86400.0
        character data_name*50

! define the local variables num_nodes_out
        num_nodes_out = np

        open(40,file='fort.40')
        rewind(40)

! output info to debug file
        write(40,*)
        write(40,*) 'get_rad:  time         = ', time
        write(40,*) 'first_call             = ', first_call
        write(40,*) 'num_nodes_out          = ', num_nodes_out

! for the first call only, initialize starting date, datasets, etc 
        if (first_call) then
        
          call get_sflux_inputs ()
          
! setup datasets
          dataset_1%name             = rad_1_file
          dataset_1%max_window_hours = rad_1_max_window_hours
          dataset_1%fail_if_missing  = rad_1_fail_if_missing
          dataset_1%relative_weight  = rad_1_relative_weight

          dataset_2%name             = rad_2_file
          dataset_2%max_window_hours = rad_2_max_window_hours
          dataset_2%fail_if_missing  = rad_2_fail_if_missing
          dataset_2%relative_weight  = rad_2_relative_weight

! output some details
          write(16,*)
          write(16,*) 'get_rad:  sflux_inputs'
          write(16,*) '  start_year             = ', start_year
          write(16,*) '  start_month            = ', start_month
          write(16,*) '  start_day              = ', start_day
          write(16,*) '  start_hour             = ', start_hour
          write(16,*) '  utc_start              = ', utc_start
          write(16,*) '  start_frac_jdate       = ', start_frac_jdate
          write(16,*) '  rad_1_file             = ', &
     &                   trim(rad_1_file)
          write(16,*) '  rad_1_max_window_hours = ', &
     &                   rad_1_max_window_hours
          write(16,*) '  rad_1_fail_if_missing  = ', &
     &                   rad_1_fail_if_missing
          write(16,*) '  rad_1_relative_weight  = ', &
     &                   rad_1_relative_weight
          write(16,*) '  rad_2_file             = ', &
     &                   trim(rad_2_file)
          write(16,*) '  rad_2_max_window_hours = ', &
     &                   rad_2_max_window_hours
          write(16,*) '  rad_2_fail_if_missing  = ', &
     &                   rad_2_fail_if_missing
          write(16,*) '  rad_2_relative_weight  = ', &
     &                   rad_2_relative_weight
          write(16,*) '  dlwrf_name             = ', &
     &                   trim(dlwrf_name)
          write(16,*) '  dswrf_name             = ', &
     &                   trim(dswrf_name)

! get basic info for dataset
          call get_dataset_info (dataset_1)
          call get_dataset_info (dataset_2)

        endif

! get the current time
        time_now = start_frac_jdate + time/secs_per_day
        write(40,*) 'current jdate        = ', time_now

! get the data at this time
        data_name = trim(dlwrf_name)
        call combine_sflux_data &
     &    (time_now, dataset_1, dataset_2, &
     &     dataset_1%exist, dataset_2%exist, &
     &     data_name, longwave_d, &
     &     num_nodes_out)
        
        data_name = trim(dswrf_name)
        call combine_sflux_data &
     &    (time_now, dataset_1, dataset_2, &
     &     dataset_1%exist, dataset_2%exist, &
     &     data_name, shortwave_d, &
     &     num_nodes_out)

! get the albedo
        write(40,*) 'calculating albedo'
        call get_albedo (albedo, num_nodes_out)

! reduce the downwards shortwave flux at the surface by the albedo
! (ensure there are no negative values from interpolation, etc)
        write(40,*) 'reducing shortwave'
        do i_node = 1, num_nodes_out
          shortwave_d(i_node) = &
     &      max( (1.0_dbl_kind - albedo(i_node))*shortwave_d(i_node), &
     &           0.0_dbl_kind )
        enddo

        close(40)

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

      return
      end
!-----------------------------------------------------------------------
      subroutine get_precip_flux (time, precip_flux)

        use global, only : dbl_kind, np
        use netcdf_io
        implicit none
        
        real(kind=dbl_kind), intent(in) :: time
        real(kind=dbl_kind), dimension(np), intent(out) :: precip_flux
        
! local variables
        integer num_nodes_out, i_node
        logical, save :: first_call = .true.
        type(dataset_info), save :: dataset_1, dataset_2
        real(kind=dbl_kind) time_now
        real(kind=dbl_kind), parameter :: secs_per_day = 86400.0
        character data_name*50

! define the local variables num_nodes_out
        num_nodes_out = np

        open(41,file='fort.41')
        rewind(41)

! output info to debug file
        write(41,*)
        write(41,*) 'get_precip_flux:  time = ', time
        write(41,*) 'first_call             = ', first_call
        write(41,*) 'num_nodes_out          = ', num_nodes_out

! for the first call only, initialize starting date, datasets, etc 
        if (first_call) then
        
          call get_sflux_inputs ()
          
! setup datasets
          dataset_1%name             = prc_1_file
          dataset_1%max_window_hours = prc_1_max_window_hours
          dataset_1%fail_if_missing  = prc_1_fail_if_missing
          dataset_1%relative_weight  = prc_1_relative_weight

          dataset_2%name             = prc_2_file
          dataset_2%max_window_hours = prc_2_max_window_hours
          dataset_2%fail_if_missing  = prc_2_fail_if_missing
          dataset_2%relative_weight  = prc_2_relative_weight

! output some details
          write(16,*)
          write(16,*) 'get_precip_flux:  sflux_inputs'
          write(16,*) '  start_year             = ', start_year
          write(16,*) '  start_month            = ', start_month
          write(16,*) '  start_day              = ', start_day
          write(16,*) '  start_hour             = ', start_hour
          write(16,*) '  utc_start              = ', utc_start
          write(16,*) '  start_frac_jdate       = ', start_frac_jdate
          write(16,*) '  prc_1_file             = ', &
     &                   trim(prc_1_file)
          write(16,*) '  prc_1_max_window_hours = ', &
     &                   prc_1_max_window_hours
          write(16,*) '  prc_1_fail_if_missing  = ', &
     &                   prc_1_fail_if_missing
          write(16,*) '  prc_1_relative_weight  = ', &
     &                   prc_1_relative_weight
          write(16,*) '  prc_2_file             = ', &
     &                   trim(prc_2_file)
          write(16,*) '  prc_2_max_window_hours = ', &
     &                   prc_2_max_window_hours
          write(16,*) '  prc_2_fail_if_missing  = ', &
     &                   prc_2_fail_if_missing
          write(16,*) '  prc_2_relative_weight  = ', &
     &                   prc_2_relative_weight
          write(16,*) '  prate_name             = ', &
     &                   trim(prate_name)

! get basic info for dataset
          call get_dataset_info (dataset_1)
          call get_dataset_info (dataset_2)

        endif

! get the current time
        time_now = start_frac_jdate + time/secs_per_day
        write(41,*) 'current jdate        = ', time_now

! get the data at this time
        data_name = trim(prate_name)
        call combine_sflux_data &
     &    (time_now, dataset_1, dataset_2, &
     &     dataset_1%exist, dataset_2%exist, &
     &     data_name, precip_flux, &
     &     num_nodes_out)
        
        close(41)

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

      return
      end
!-----------------------------------------------------------------------
      subroutine get_dataset_info (info)

        use global, only : dbl_kind, np, xlon, ylat
        use netcdf_io
        implicit none
        type(dataset_info), intent(inout) :: info

        character file_name*50, get_file_name*50, data_name*50
        integer file_num, alloc_stat, num_nodes_out

! define num_nodes_out (number of nodes in model grid)
        num_nodes_out = np

! determine if the first file exists for this dataset
        file_num = 1
        file_name = get_file_name(info%name, file_num)
        inquire(file=file_name, exist=info%exist)
        
        write(16,*)
        write(16,*) 'netCDF dataset and existence: ', &
     &              trim(info%name), info%exist
        write(16,*)

! run should fail if dataset doesn't exist and fail_if_missing is set
        if ( (.not. info%exist) .and. (info%fail_if_missing) ) then
          call halt_error ('missing dataset: ' // info%name)
        endif

! if this dataset exists, then get additional info
        if (info%exist) then
          write(16,*) 'getting additional info for: ', info%name
          
          call get_times_etc (info%name, info%times, &
     &                        info%file_num_for_time, &
     &                        info%time_num_for_time, &
     &                        info%num_times, &
     &                        info%num_files, info%jdate_for_file, &
     &                        info%nx, info%ny, max_times, max_files)

! allocate memory for lon and lat
          allocate (info%lon(info%nx,info%ny), &
     &              info%lat(info%nx,info%ny), &
     &              stat=alloc_stat)
          call check_allocation('lon/lat', 'get_dataset_info', &
     &                          alloc_stat)

! read in lon and lat
          data_name = 'lon'
          call read_coord (file_name, data_name, info%lon, &
     &                     info%nx, info%ny)

          data_name = 'lat'
          call read_coord (file_name, data_name, info%lat, &
     &                     info%nx, info%ny)

! confine lon to -180->180 range, convert lon/lat to radians
          call fix_coords (info%lon, info%lat, info%nx, info%ny)

! get the number of nodes and elements for the data grid
          info%num_nodes = info%nx * info%ny
          info%num_elems = (info%nx - 1) * (info%ny - 1) * 2

!         write(*,*)
!         write(*,*) info%name, info%num_nodes, info%num_elems
          
! allocate memory for grid conversion variables
          allocate (info%node_i(info%num_nodes), &
     &              info%node_j(info%num_nodes), &
     &              info%node_num(info%nx,info%ny), &
     &              info%elem_nodes(info%num_elems,3), &
     &              info%in_elem_for_out_node(num_nodes_out), &
     &              stat=alloc_stat)
          call check_allocation('integer grid variables', &
     &                          'get_dataset_info', alloc_stat)

          allocate (info%weight(num_nodes_out,3), &
     &              stat=alloc_stat)
          call check_allocation('real grid variables', &
     &                          'get_dataset_info', alloc_stat)

! create list of all nodes for this grid
          call list_nodes (info%node_i, info%node_j, info%node_num, &
     &                     info%num_nodes, info%nx, info%ny)

! create list of all elements for this grid
          call list_elems (info%elem_nodes, info%node_num, &
     &                     info%nx, info%ny, info%num_elems)

! calculate the weightings from data grid to model nodes
          call get_weight (info%lon, info%lat, xlon, ylat, &
     &                     info%elem_nodes, info%node_i, info%node_j, &
     &                     info%nx, info%ny, info%num_elems, &
     &                     info%num_nodes, &
     &                     num_nodes_out, info%in_elem_for_out_node, &
     &                     info%weight)

        endif

      return
      end
!-----------------------------------------------------------------------
      character*3 function char_num (num)
        implicit none
        integer, intent(in) :: num
        character char*3
        
10      format ('00', i1)
20      format ('0', i2)
30      format (i3)

        if (num .le. 9) then
          write(char,10) num
        else if (num .le. 99) then
          write(char,20) num
        else if (num .le. 999) then
          write(char,30) num
        else
          call halt_error ('get_char_num: num too large!')
        endif
        
        char_num = char

      return
      end
!-----------------------------------------------------------------------
      character*50 function get_file_name (dataset_name, num)
        implicit none
        integer, intent(in) :: num
        character, intent(in) ::  dataset_name*50

        character char_num*3
        character, parameter :: prefix*6 = 'sflux/'
        character, parameter :: suffix*3 = '.nc'
        
        get_file_name = prefix // trim(dataset_name) // '.' // &
     &                  char_num(num) // suffix

      return
      end
!-----------------------------------------------------------------------
      subroutine check_err(iret)
        implicit none
        integer iret
        include 'netcdf.inc'
        if (iret .ne. NF_NOERR) then
          call halt_error (nf_strerror(iret))
        endif
      return
      end
!-----------------------------------------------------------------------
      INTEGER FUNCTION JD(YYYY,MM,DD)
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: YYYY,MM,DD
!              DATE ROUTINE JD(YYYY,MM,DD) CONVERTS CALENDER DATE TO
!              JULIAN DATE.  SEE CACM 1968 11(10):657, LETTER TO THE
!              EDITOR BY HENRY F. FLIEGEL AND THOMAS C. VAN FLANDERN.
!    EXAMPLE JD(1970,1,1)=2440588
      JD=DD-32075+1461*(YYYY+4800+(MM-14)/12)/4 &
     &         +367*(MM-2-((MM-14)/12)*12)/12-3* &
     &         ((YYYY+4900+(MM-14)/12)/100)/4
      RETURN
      END
!-----------------------------------------------------------------------
      subroutine get_times_etc (dataset_name, times, &
     &                          file_num_for_time, &
     &                          time_num_for_time, &
     &                          num_times, &
     &                          num_files, jdate_for_file, &
     &                          nx, ny, max_times, max_files)

        use global, only : dbl_kind
        implicit none

        character, intent(in) ::  dataset_name*50
        integer, intent(in) :: max_times, max_files
        integer, intent(out) :: num_times, num_files, nx, ny
        real(kind=dbl_kind), intent(out), dimension(max_times) :: times
        integer, intent(out), dimension(max_times) :: &
     &    file_num_for_time, time_num_for_time
        integer, intent(out), dimension(max_files) :: jdate_for_file
        
        integer file_num, num_file_times, nx_test, ny_test
        logical have_file, repeat, at_end
        character file_name*50, get_file_name*50
        integer, parameter :: max_file_times = 1000
        real(kind=dbl_kind) test_time, file_times(max_file_times)
        integer i_time, repeat_num

! determine how many files there are
        file_num = 0
        do
          file_num = file_num + 1
          file_name = get_file_name(dataset_name, file_num)
          inquire(file=file_name, exist=have_file)
          if (.not. have_file) exit
          num_files = file_num
        enddo

! ensure that num_files doesn't exceed max_files
        if (num_files .gt. max_files) then
          call halt_error ('num_files exceeds max_files!')
        endif

! get the dimensions of the first of the files
        file_num = 1
        file_name = get_file_name(dataset_name, file_num)
        call get_dims (file_name, nx, ny)

! loop over files
        do file_num = 1, num_files

! get the name of this file
          file_name = get_file_name(dataset_name, file_num)

! make sure that the physical dimensions (nx and ny) are the same
          call get_dims (file_name, nx_test, ny_test)
          if ( (nx_test .ne. nx) .or. (ny_test .ne. ny) ) then
            call halt_error ('nx and/or ny mismatch!')
          endif

! get the times in this file
          call get_file_times (file_name, file_times, &
     &                         jdate_for_file(file_num), &
     &                         max_file_times, num_file_times)

! check that num_file_times does not exceed max_times
          if (num_file_times .gt. max_times) then
            call halt_error ('num_file_times exceeds max_times!')
          endif

! if this is the first file, then store all file_times in the overall
! time vector (add jdate_for_file)
          if (file_num .eq. 1) then
            do i_time = 1, num_file_times
              times(i_time) = real(jdate_for_file(file_num),dbl_kind) &
     &                      + file_times(i_time)
              file_num_for_time(i_time) = file_num
              time_num_for_time(i_time) = i_time
            enddo
            num_times = num_file_times

! if this is not the first file, then loop over file_times, and add them
! one at a time _if_ they're not duplicates
          else
            do i_time = 1, num_file_times

              test_time = real(jdate_for_file(file_num),dbl_kind) &
     &                  + file_times(i_time)

              call check_times (test_time, times, num_times, &
     &                          repeat_num, repeat, at_end)

! if this time is a repeat, then use this file for this time
              if (repeat) then
                file_num_for_time(repeat_num) = file_num
                time_num_for_time(repeat_num) = i_time

! if this is not a repeat, and is at the end, then add it to the list
! (but first check to make sure max_times isn't exceeded)
              else if (at_end) then

                num_times = num_times + 1
                if (num_times .gt. max_times) then
                  call halt_error ('num_times exceeds max_times!')
                endif
                
                times(num_times) = test_time
                file_num_for_time(num_times) = file_num
                time_num_for_time(num_times) = i_time

              endif

            enddo

          endif

        enddo
        
        write(16,*) 'num_files = ', num_files
        write(16,*) 'num_times = ', num_times
        write(16,*) 'first time = ', times(1)
        write(16,*) 'last  time = ', times(num_times)

!       write(*,*) dataset_name, num_files, num_times
!       do i_time = 1, num_times
!         write(*,*) i_time, file_num_for_time(i_time), &
!    &                       time_num_for_time(i_time), times(i_time)
!       enddo
        
      return
      end
!-----------------------------------------------------------------------
      subroutine get_file_times (file_name, file_times, &
     &                           file_julian_date, max_file_times, &
     &                           num_file_times)

        use global, only : dbl_kind
        implicit none
        include 'netcdf.inc'

        character, intent(in) ::  file_name*50
        integer, intent(in) :: max_file_times
        integer, intent(out) :: num_file_times, file_julian_date
        real(kind=dbl_kind), intent(out), dimension(max_file_times) :: &
     &    file_times

! file_times_tmp must be default real (netcdf)
        real, dimension(max_file_times) :: file_times_tmp
        
        integer ncid, iret, time_dim, time_id, i_time
        character data_name*50, attr_name*50
        integer, allocatable, dimension(:) :: base_date
        integer day, month, year, jd, n_base_date, allocate_stat

! open file_name and enter read-only mode
        iret = nf_open(file_name, NF_NOWRITE, ncid)
        call check_err(iret)

! get the variable id for the time variable
        data_name = 'time'
        iret = nf_inq_varid(ncid, data_name, time_id)
        call check_err(iret)

! get the time dimension id
        iret = nf_inq_vardimid (ncid, time_id, time_dim)
        call check_err(iret)
        
! determine number of times stored in the time dimension
        iret = nf_inq_dimlen(ncid, time_dim, num_file_times)
        call check_err(iret)

! read the time vector for this file
        iret = nf_get_var_real(ncid, time_id, file_times_tmp)
        call check_err(iret)

! convert from file real type to data real type
        do i_time = 1, num_file_times
          file_times(i_time) = real(file_times_tmp(i_time),dbl_kind)
        enddo

! get the base_date - the time and date which corresponds with time zero
        attr_name = 'base_date'

! get size of base_date (make sure it is at least 3)
        iret = nf_inq_attlen (ncid, time_id, attr_name, n_base_date)
        if (n_base_date .lt. 3) then
          call halt_error ('insufficient fields in base_date!')
        endif

! allocate space for base_date
        allocate(base_date(n_base_date), stat=allocate_stat)
        call check_allocation('base_date', 'get_file_times', &
     &                        allocate_stat)

! read base_date
        iret = nf_get_att_int(ncid, time_id, attr_name, base_date)

! convert base_date to fractional Julian date
        year = base_date(1)
        month = base_date(2)
        day = base_date(3)
        file_julian_date = jd(year,month,day)

! deallocate base_date
        deallocate(base_date)

! close the netCDF file
        iret = nf_close(ncid)
        call check_err(iret)

      return
      end
!-----------------------------------------------------------------------
      subroutine check_times (test_time, times, num_times, &
     &                        repeat_num, repeat, at_end)

        use global, only : dbl_kind
        implicit none

        integer, intent(in) :: num_times
        real(kind=dbl_kind), intent(in) :: test_time
        real(kind=dbl_kind), intent(in), dimension(num_times) :: times
        logical, intent(out) :: repeat, at_end
        integer, intent(out) :: repeat_num

        real(kind=dbl_kind), parameter :: time_eps = 0.001
        integer i_time
        
        repeat = .false.
        do i_time = 1, num_times
          if (abs(test_time - times(i_time)) .le. time_eps) then
            repeat = .true.
            repeat_num = i_time
          endif
        enddo
        
        at_end = ((test_time - (times(num_times) + time_eps)) .gt. 0.0)
        
      return
      end
!-----------------------------------------------------------------------
      subroutine get_dims (file_name, nx, ny)
        implicit none
        include 'netcdf.inc'
        character, intent(in) ::  file_name*50
        integer, intent(out) :: nx, ny
        
        integer ncid, iret, nx_dim, ny_dim, dim_ids(3), test_var_id
        character, parameter :: test_variable*50 = 'lat'

! open file_name and enter read-only mode
        iret = nf_open(file_name, NF_NOWRITE, ncid)
        call check_err(iret)

! get the variable ID for the test variable
        iret = nf_inq_varid(ncid, test_variable, test_var_id)
        call check_err(iret)

! get the dimension IDs for the test variable
        iret = nf_inq_vardimid (ncid, test_var_id, dim_ids)
        call check_err(iret)
        
! determine number of points in the nx and ny dimensions
        nx_dim = dim_ids(1)
        iret = nf_inq_dimlen(ncid, nx_dim, nx)
        call check_err(iret)

        ny_dim = dim_ids(2)
        iret = nf_inq_dimlen(ncid, ny_dim, ny)
        call check_err(iret)

! close the netCDF file
        iret = nf_close(ncid)
        call check_err(iret)
        
      return
      end
!-----------------------------------------------------------------------
      subroutine halt_error (message)
        implicit none
        character(*), intent(in) :: message

        write(11,*)
        write(11,*) message
        stop

      return
      end
!-----------------------------------------------------------------------
      subroutine read_coord (file_name, data_name, coord, &
     &                       nx, ny)

        use global, only : dbl_kind
        implicit none
        include 'netcdf.inc'

        character, intent(in) ::  file_name*50, data_name*50
        integer, intent(in) :: nx, ny
        real(kind=dbl_kind), intent(out), dimension(nx,ny) :: coord
        
        integer iret, ncid, var_id, i, j
! coord_tmp must be default real (netcdf)
        real coord_tmp(nx,ny)

! open file_name and enter read-only mode
        iret = nf_open(file_name, NF_NOWRITE, ncid)
        call check_err(iret)

! get the variable id for this variable
        iret = nf_inq_varid(ncid, data_name, var_id)
        call check_err(iret)

! read the coordinate data
        iret = nf_get_var_real (ncid, var_id, coord_tmp)
        call check_err(iret)

! convert from file real type to data real type
        do j = 1, ny
          do i = 1, nx
            coord(i,j) = real(coord_tmp(i,j),dbl_kind)
          enddo
        enddo

! close the netCDF file
        iret = nf_close(ncid)
        call check_err(iret)
        
      return
      end
!-----------------------------------------------------------------------
      subroutine read_data (file_name, data_name, data, &
     &                      nx, ny, time_num)

        use global, only : dbl_kind
        implicit none
        include 'netcdf.inc'

        character(*), intent(in) ::  file_name, data_name
        integer, intent(in) :: nx, ny, time_num
        real(kind=dbl_kind), intent(out), dimension(nx,ny) :: data
        
        integer iret, ncid, var_id, i, j
        integer data_start(3), data_count(3)
        
! data_tmp must be default real (netcdf)
        real data_tmp(nx,ny)

! specify size of read
        data_start(1) = 1
        data_start(2) = 1
        data_start(3) = time_num
        data_count(1) = nx
        data_count(2) = ny
        data_count(3) = 1

! open file_name and enter read-only mode
        iret = nf_open(file_name, NF_NOWRITE, ncid)
        call check_err(iret)

! get the variable id for this variable
        iret = nf_inq_varid(ncid, data_name, var_id)
        call check_err(iret)

! read the data
        iret = nf_get_vara_real(ncid, var_id, data_start, &
     &                          data_count, data_tmp)
        call check_err(iret)

! convert from file real type to data real type
        do j = 1, ny
          do i = 1, nx
            data(i,j) = real(data_tmp(i,j),dbl_kind)
          enddo
        enddo

! close the netCDF file
        iret = nf_close(ncid)
        call check_err(iret)
        
      return
      end
!-----------------------------------------------------------------------
      subroutine list_nodes (node_i, node_j, node_num, &
     &                       num_nodes, nx, ny)
        implicit none
        
        integer, intent(in) :: nx, ny, num_nodes
        integer, intent(out), dimension(num_nodes) :: node_i, node_j
        integer, intent(out), dimension(nx,ny) :: node_num
        
        integer i_node, i, j
        
        i_node = 0
        do j = 1, ny
          do i = 1, nx
            i_node = i_node + 1
            node_i(i_node) = i
            node_j(i_node) = j
            node_num(i,j) = i_node
          enddo
        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine list_elems (elem_nodes, node_num, &
     &                       nx, ny, num_elems)
        implicit none
        integer, intent(in) :: nx, ny, num_elems
        integer, intent(in), dimension(nx,ny) :: node_num
        integer, intent(out), dimension(num_elems,3) :: elem_nodes
        
        integer i, j, i_elem

        i_elem = 0
        do j = 1, ny-1
          do i = 1, nx-1

! define the first element in this grid box
            i_elem = i_elem + 1
            elem_nodes(i_elem,1) = node_num(i,j)
            elem_nodes(i_elem,2) = node_num(i+1,j+1)
            elem_nodes(i_elem,3) = node_num(i,j+1)

! define the second element in this grid box
            i_elem = i_elem + 1
            elem_nodes(i_elem,1) = node_num(i,j)
            elem_nodes(i_elem,2) = node_num(i+1,j)
            elem_nodes(i_elem,3) = node_num(i+1,j+1)

          enddo
        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine get_weight (x_in, y_in, x_out, y_out, &
     &                       elem_nodes, node_i, node_j, &
     &                       nx, ny, num_elems, num_nodes, &
     &                       num_nodes_out, in_elem_for_out_node, &
     &                       weight)

        use global, only : dbl_kind
        implicit none

        integer, intent(in) :: nx, ny, num_elems, num_nodes
        integer, intent(in) :: num_nodes_out
        integer, intent(in), dimension(num_nodes) :: node_i, node_j
        integer, intent(in), dimension(num_elems,3) :: elem_nodes
        real(kind=dbl_kind), intent(in), dimension(nx,ny) :: x_in, y_in
        real(kind=dbl_kind), intent(in), dimension(num_nodes_out) :: &
     &    x_out, y_out
        real(kind=dbl_kind), intent(out), &
     &    dimension(num_nodes_out,3) :: weight
        integer, intent(out), dimension(num_nodes_out) :: &
     &    in_elem_for_out_node

        real(kind=dbl_kind) area_in(num_elems)
        integer i_elem, i_node
        integer i1, j1, i2, j2, i3, j3
        integer last_elem, top, floor
        real(kind=dbl_kind) x1, y1, x2, y2, x3, y3, x4, y4
        real(kind=dbl_kind) a1, a2, a3, aa, ae
        real(kind=dbl_kind) ae_min
        real(kind=dbl_kind), parameter :: epsilon = 1.0d-10
        real(kind=dbl_kind), parameter :: bad_point_flag = -9.9d99
        logical zero_ae, completed_check

! calculate and store the areas of the input grid elements
        do i_elem = 1, num_elems

          i1 = node_i(elem_nodes(i_elem,1))
          j1 = node_j(elem_nodes(i_elem,1))
          x1 = x_in(i1,j1)
          y1 = y_in(i1,j1)

          i2 = node_i(elem_nodes(i_elem,2))
          j2 = node_j(elem_nodes(i_elem,2))
          x2 = x_in(i2,j2)
          y2 = y_in(i2,j2)

          i3 = node_i(elem_nodes(i_elem,3))
          j3 = node_j(elem_nodes(i_elem,3))
          x3 = x_in(i3,j3)
          y3 = y_in(i3,j3)

          area_in(i_elem) = 0.5 * &
     &                      ( (x1-x3)*(y2-y3) + (x3-x2)*(y1-y3) )

        enddo

! now loop over the nodes of the output grid, searching for the
! surrounding elements on the input grid
!
! for first output node, begin search with first input element
        last_elem = 0
        top = 1
        floor = 1

        do i_node = 1, num_nodes_out

          ae_min = 1.0e25
          in_elem_for_out_node(i_node) = 0

! initialize flag which indicates that we've found correct element
! (where ae .eq. 0)
          zero_ae = .false.

! search for element, starting with location of previous node
100       continue

! this block implements logic which tells which is the next element
! to consider

! first output node, first input element
            if (last_elem .eq. 0) then
              i_elem = 1

! first iteration with new output node
! (begin search at correct element for previous node)
            else if ( (top .eq. 0) .and. (floor .eq. 0) ) then
              i_elem = last_elem
              top = i_elem
              floor = i_elem

! if we searched the "floor" value last
            else if (last_elem .eq. floor) then
              if (top .eq. num_elems) then
                i_elem = floor - 1
                floor = floor - 1
              else
                i_elem = top + 1
                top = top + 1
              endif

! if we searched the "top" value last
            else if (last_elem .eq. top) then
              if (floor .eq. 1) then
                i_elem = top + 1
                top = top + 1
              else
                i_elem = floor - 1
                floor = floor - 1
              endif

! error in logic somehow (shouldn't happen, unless I screwed up code)
            else
              write(11,*)
              write(11,*) 'search error in get_weight!'
              stop
            endif

! get the locations of the nodes for this element on the input grid
            i1 = node_i(elem_nodes(i_elem,1))
            j1 = node_j(elem_nodes(i_elem,1))
            x1 = x_in(i1,j1)
            y1 = y_in(i1,j1)

            i2 = node_i(elem_nodes(i_elem,2))
            j2 = node_j(elem_nodes(i_elem,2))
            x2 = x_in(i2,j2)
            y2 = y_in(i2,j2)

            i3 = node_i(elem_nodes(i_elem,3))
            j3 = node_j(elem_nodes(i_elem,3))
            x3 = x_in(i3,j3)
            y3 = y_in(i3,j3)
!           write(*,*) i_elem, x1, x2, x3
!           write(*,*) i_elem, y1, y2, y3

! get the locations of this node for the output grid
            x4 = x_out(i_node)
            y4 = y_out(i_node)

! calculate some type of areas
            a1 = (x4-x3)*(y2-y3) + (x2-x3)*(y3-y4)
            a2 = (x4-x1)*(y3-y1) - (y4-y1)*(x3-x1)
            a3 = (y4-y1)*(x2-x1) - (x4-x1)*(y2-y1)
            aa = abs(a1) + abs(a2) + abs(a3)
            if (area_in(i_elem) .gt. 0.0) then
              ae = abs(aa - 2.0*area_in(i_elem)) &
     &           / (2.0*area_in(i_elem))
            else
              ae = 1.0e25
            endif

! if ae equals zero (within epsilon) then we've found correct element
            zero_ae = (ae .lt. epsilon)

! determine if this is the smallest ae
! (element of input grid that contains node for output grid)
            if (ae .lt. ae_min) then
              ae_min = ae
              in_elem_for_out_node(i_node) = i_elem
            endif

! have we checked all of the elements?
            completed_check = (floor .eq. 1) .and. (top .eq. num_elems)

! change last_elem pointer
            last_elem = i_elem

! loop again if need to
          if ( (.not. completed_check) .and. (.not. zero_ae) ) goto 100

! if we didnt find a good ae_min, then there are problems
          if (in_elem_for_out_node(i_node) .eq. 0) then
            write(11,*)
            write(11,*) 'Couldn''t find suitable element in input grid'
            write(11,*) 'for output node #', i_node
            write(11,*) 'x_out, y_out = ', x_out(i_node), y_out(i_node)
            write(11,*) 'in_elem_for_out_node, ae_min = ', &
     &                   in_elem_for_out_node(i_node), ae_min
            stop
          endif

! the proper element has been found, now calculate the averaging weights
! (but make sure that there are no bad points from get_xy)

! get the locations of the nodes for this element on the input grid
          i_elem = in_elem_for_out_node(i_node)
          i1 = node_i(elem_nodes(i_elem,1))
          j1 = node_j(elem_nodes(i_elem,1))
          x1 = x_in(i1,j1)
          y1 = y_in(i1,j1)

          i2 = node_i(elem_nodes(i_elem,2))
          j2 = node_j(elem_nodes(i_elem,2))
          x2 = x_in(i2,j2)
          y2 = y_in(i2,j2)

          i3 = node_i(elem_nodes(i_elem,3))
          j3 = node_j(elem_nodes(i_elem,3))
          x3 = x_in(i3,j3)
          y3 = y_in(i3,j3)

! check to make sure none of locations has been flagged as bad
          if ( (x1 .le. bad_point_flag) .or. &
     &         (x2 .le. bad_point_flag) .or. &
     &         (x3 .le. bad_point_flag) ) then
            write(11,*)
            write(11,*) 'Bad x-y flag from input grid'
            write(11,*) 'for output node #', i_node
            write(11,*) 'x_out, y_out = ', x_out(i_node), y_out(i_node)
            write(11,*) 'in_elem_for_out_node, ae_min = ', &
     &                   in_elem_for_out_node(i_node), ae_min
            stop
          endif

! get the locations of this node for the output grid
          x4 = x_out(i_node)
          y4 = y_out(i_node)

! now calculate the weighting functions
          weight(i_node,1) = ( (x4-x3)*(y2-y3) + (x2-x3)*(y3-y4) ) &
     &                     / ( 2.0*area_in(i_elem) )
          weight(i_node,2) = ( (x4-x1)*(y3-y1) - (y4-y1)*(x3-x1) ) &
     &                     / ( 2.0*area_in(i_elem) )
          weight(i_node,3) = ( -(x4-x1)*(y2-y1) + (y4-y1)*(x2-x1) ) &
     &                     / ( 2.0*area_in(i_elem) )

! this node is done, reset top and floor so next iteration is informed
          top = 0
          floor = 0

! debugging outputs
!999      format(2(i8,1x),g14.5,1x,4(f10.5,1x))
!         if (mod(i_node-1,1000) .eq. 0) then
!           write(50,*)
!           write(50,999) i_node, in_elem_for_out_node(i_node), ae_min, &
!    &           weight(i_node,1), weight(i_node,2), weight(i_node,3), &
!    &           weight(i_node,1) + weight(i_node,2) + weight(i_node,3)
!           write(50,*) 'x_out, y_out = ', x_out(i_node), y_out(i_node)
!           write(50,*) '   x1,    y1 = ', x1, y1
!           write(50,*) '   x2,    y2 = ', x2, y2
!           write(50,*) '   x3,    y3 = ', x3, y3
!         endif

        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine fix_coords (lon, lat, nx, ny)

        use global, only : dbl_kind
        implicit none

        integer, intent(in) :: nx, ny
        real(kind=dbl_kind), intent(inout), dimension(nx,ny) :: &
     &    lon, lat

        real(kind=dbl_kind) pi, deg_to_rad
        integer i, j

        pi = 4.0 * atan(1.0)
        deg_to_rad = pi / 180.0

! confine lon to -180->180 range
        do j = 1, ny
          do i = 1, nx
            if (lon(i,j) .gt. 180.0) lon(i,j) = lon(i,j) - 360.0
          enddo
        enddo

! convert degrees to radians
        do j = 1, ny
          do i = 1, nx
            lon(i,j) = lon(i,j) * deg_to_rad
            lat(i,j) = lat(i,j) * deg_to_rad
          enddo
        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine get_sflux_inputs ()

        use global, only : dbl_kind
        use netcdf_io
        implicit none
        
        character, parameter :: &
     &    sflux_inputs_file*50 = 'sflux/sflux_inputs.txt'
        real(kind=dbl_kind), parameter :: hours_per_day = 24.0
        integer jd
        logical, save :: first_call = .true.
        logical exst

! we only need to do this the first time this subroutine is called
        if (first_call) then
          first_call = .false.

! make sure sflux_inputs_file exists (fail if it doesn't)
          inquire(file=sflux_inputs_file, exist=exst)
          if (.not. exst) &
     &      call halt_error ('you must have sflux_inputs_file!')

! open input deck, and read in namelist
          open (unit=77, file=sflux_inputs_file, status='old')
          read(77, nml=sflux_inputs)
          close (unit = 77)
!         write(*,nml=sflux_inputs)

! validate the inputs
          if (start_year .lt. -9000) call halt_error &
     &      ('sflux_inputs_file: you must supply a value for start_year')
          if (start_month .lt. -9000) call halt_error &
     &      ('sflux_inputs_file: you must supply a value for start_month')
          if (start_day .lt. -9000) call halt_error &
     &      ('sflux_inputs_file: you must supply a value for start_day')
          if (start_hour .lt. -9000.0) call halt_error &
     &      ('sflux_inputs_file: you must supply a value for start_hour')
          if (utc_start .lt. -9000.0) call halt_error &
     &      ('sflux_inputs_file: you must supply a value for utc_start')

! calculate the starting Julian date and starting fractional Julian date
          start_jdate = jd(start_year,start_month,start_day)

          start_frac_jdate = real(start_jdate,dbl_kind) &
     &                     + (start_hour + utc_start) / hours_per_day
     
        endif  !  first_call

      return
      end
!-----------------------------------------------------------------------
      subroutine get_bracket (time, input_times, time_num_1, &
     &                        got_bracket, num_times)

        use global, only : dbl_kind
        implicit none

        integer, intent(in) :: num_times
        real(kind=dbl_kind), intent(in) :: time
        real(kind=dbl_kind), intent(in), dimension(num_times) :: &
     &    input_times
        integer, intent(out) :: time_num_1
        logical, intent(out) :: got_bracket

        time_num_1 = 0
        got_bracket = .false.
        do
          time_num_1 = time_num_1 + 1
          got_bracket = ( input_times(time_num_1) .le. time .and. &
     &                    time .le. input_times(time_num_1+1) )
          if (got_bracket .or. (time_num_1 .eq. num_times-1)) exit
        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine get_sflux_data (time_now, info, data_name, &
     &                           data_out, got_suitable_bracket, &
     &                           num_nodes_out)

        use global, only : dbl_kind
        use netcdf_io
        implicit none

        real(kind=dbl_kind), intent(in) :: time_now
        type(dataset_info), intent(in) :: info
        character(*), intent(in) :: data_name
        integer, intent(in) :: num_nodes_out
        real(kind=dbl_kind), intent(out), dimension(num_nodes_out) :: &
     &    data_out
        logical, intent(out) :: got_suitable_bracket
        
        real(kind=dbl_kind), parameter :: hours_per_day = 24.0
        real(kind=dbl_kind), dimension(info%nx,info%ny) :: data_tmp
        real(kind=dbl_kind) window_hours
        integer time_num_1, time_num_2
        logical got_bracket

! find bracketing times for the current time
        call get_bracket (time_now, info%times, time_num_1, &
     &                    got_bracket, info%num_times)
     
! check to see whether we've exceeded the max_window
        window_hours = hours_per_day * ( info%times(time_num_1+1) - &
     &                                   info%times(time_num_1) )
        got_suitable_bracket = got_bracket .and. &
     &                         (window_hours .lt. info%max_window_hours)

! fail if we don't have a suitable bracket and if fail_if_missing is set
        if ( (.not. got_suitable_bracket) .and. &
     &       (info%fail_if_missing) ) then

          write(11,*)
          write(11,*) 'no appropriate time exists for: ', info%name
          write(11,*) 'time_now = ', time_now
          write(11,*) 'first time available = ', &
     &               info%times(1)
          write(11,*) 'last time available = ', &
     &               info%times(info%num_times)
          write(11,*) 'got_bracket = ', got_bracket
          write(11,*) 'got_suitable_bracket = ', got_suitable_bracket

          if (got_bracket) then
            write(11,*) 'window_hours = ', window_hours
            write(11,*) 'max_window_hours = ', info%max_window_hours
            write(11,*) 'time_1 = ', info%times(time_num_1)
            write(11,*) 'time_2 = ', info%times(time_num_1+1)
          endif
          stop

        endif

! if we do have a suitable bracket, then read in the bracketing data
! and interpolate in time to the current time
! (but still on original grid)
        if (got_suitable_bracket) then

          time_num_2 = time_num_1 + 1

          call interp_time &
     &      (info%name, time_now, data_name, &
     &       info%times(time_num_1), &
     &       info%file_num_for_time(time_num_1), &
     &       info%time_num_for_time(time_num_1), & 
     &       info%times(time_num_2), &
     &       info%file_num_for_time(time_num_2), &
     &       info%time_num_for_time(time_num_2), &
     &       data_tmp, info%nx, info%ny )
          
! now the data needs to be interpolated in space from the original data
! grid to the model grid
          call interp_grid (data_out, data_tmp, info%weight, &
     &                      info%in_elem_for_out_node, &
     &                      info%elem_nodes, info%num_elems, &
     &                      info%node_i, info%node_j, &
     &                      info%num_nodes, num_nodes_out, &
     &                      info%nx, info%ny)

        endif

      return
      end
!-----------------------------------------------------------------------
      subroutine interp_time (dataset_name, time, data_name, &
     &                        time_1, file_num_1, file_time_1, &
     &                        time_2, file_num_2, file_time_2, &
     &                        data_out, nx, ny)

        use global, only : dbl_kind
        implicit none

        integer, intent(in) :: file_num_1, file_time_1, &
     &                         file_num_2, file_time_2, &
     &                         nx, ny
        character(*), intent(in) :: dataset_name, data_name
        real(kind=dbl_kind), intent(in) :: time, time_1, time_2
        real(kind=dbl_kind), intent(out), dimension(nx,ny) :: data_out

        character file_name_1*50, file_name_2*50, get_file_name*50
        real(kind=dbl_kind), dimension(nx,ny) :: data_1, data_2
        real(kind=dbl_kind) time_ratio
        integer i, j

        file_name_1 = get_file_name(dataset_name, file_num_1)
        file_name_2 = get_file_name(dataset_name, file_num_2)
        
        call read_data (file_name_1, data_name, data_1, &
     &                  nx, ny, file_time_1)

        call read_data (file_name_2, data_name, data_2, &
     &                  nx, ny, file_time_2)

        time_ratio = (time - time_1) / (time_2 - time_1)
        
        do j = 1, ny
          do i = 1, nx
            data_out(i,j) = data_1(i,j) &
     &                    + (data_2(i,j) - data_1(i,j)) * time_ratio
          enddo
        enddo
        
      return
      end
!-----------------------------------------------------------------------
      subroutine interp_grid (data_out, data_in, weight, &
     &                        in_elem_for_out_node, &
     &                        elem_nodes, num_elems, &
     &                        node_i, node_j, &
     &                        num_nodes, num_nodes_out, &
     &                        nx, ny)

        use global, only : dbl_kind
        implicit none

        integer, intent(in) :: num_nodes, num_nodes_out, num_elems
        integer, intent(in) :: nx, ny
        real(kind=dbl_kind), intent(in), dimension(num_nodes_out,3) :: &
     &    weight
        integer, intent(in), dimension(num_nodes) :: node_i, node_j
        real(kind=dbl_kind), intent(in), dimension(nx,ny) :: data_in
        real(kind=dbl_kind), intent(out), dimension(num_nodes_out) :: &
     &    data_out
        integer, intent(in), dimension(num_elems,3) :: elem_nodes
        integer, intent(in), dimension(num_nodes_out) :: &
     &    in_elem_for_out_node
        
        integer i_node, i_elem, i1, i2, i3, j1, j2, j3

! loop over the output nodes
        do i_node = 1, num_nodes_out

! get the locations of the nodes for the surrounding element on the
! input grid
          i_elem = in_elem_for_out_node(i_node)
          i1 = node_i(elem_nodes(i_elem,1))
          j1 = node_j(elem_nodes(i_elem,1))
          i2 = node_i(elem_nodes(i_elem,2))
          j2 = node_j(elem_nodes(i_elem,2))
          i3 = node_i(elem_nodes(i_elem,3))
          j3 = node_j(elem_nodes(i_elem,3))

! the data on the output grid is simply the weighted data from the input
! grid
          data_out(i_node) = data_in(i1,j1) * weight(i_node,1) &
     &                     + data_in(i2,j2) * weight(i_node,2) &
     &                     + data_in(i3,j3) * weight(i_node,3)

        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine combine_sflux_data (time_now, info_1, info_2, &
     &                               read_1, read_2, &
     &                               data_name, data_out, &
     &                               num_nodes_out)
      
        use global, only : dbl_kind
        use netcdf_io
        implicit none

        real(kind=dbl_kind), intent(in) :: time_now
        type(dataset_info), intent(in) :: info_1, info_2
        character(*), intent(in) :: data_name
        logical, intent(in) :: read_1, read_2
        integer, intent(in) :: num_nodes_out
        real(kind=dbl_kind), intent(out), dimension(num_nodes_out) :: &
     &    data_out
        
        real(kind=dbl_kind), dimension(num_nodes_out) :: data_1, data_2
        real(kind=dbl_kind) local_weight_2, sum_weights
        logical got_data_1, got_data_2, bad_node_2
        integer i_node

! set initial values for got_data_x
! (set to true only for successful reads)
        got_data_1 = .false.
        got_data_2 = .false.

! attempt to read data_1 and data_2 (only if input flags specify)
        if (read_1) then
          call get_sflux_data (time_now, info_1, data_name, &
     &                         data_1, got_data_1, num_nodes_out)
        endif

        if (read_2) then
          call get_sflux_data (time_now, info_2, data_name, &
     &                         data_2, got_data_2, num_nodes_out)
        endif

! depending on whether different datasets exist at this time, combine
! data in different ways

! if data_1 is missing, then fail
        if (.not. got_data_1) then
          call halt_error ('missing data_1: ' // data_name)

! if data_1 exists, but not data_2, then simply use data_1 (for all)
        else
          if (.not. got_data_2) then
            data_out = data_1

! if data_1 and data_2 exist, then combine
          else

! loop over output nodes, calculating combined value
            do i_node = 1, num_nodes_out

! determine if this node is within grid for data_2
              bad_node_2 = ( &
     &            info_2%weight(i_node,1) .lt. 0.0 .or. &
     &            info_2%weight(i_node,1) .gt. 1.0 .or. &
     &            info_2%weight(i_node,2) .lt. 0.0 .or. &
     &            info_2%weight(i_node,2) .gt. 1.0 .or. &
     &            info_2%weight(i_node,3) .lt. 0.0 .or. &
     &            info_2%weight(i_node,3) .gt. 1.0 &
     &                 )

! if this is a bad node for data_2, then don't weight data_2
              if (bad_node_2) then
                local_weight_2 = 0.0
              else
                local_weight_2 = info_2%relative_weight
              endif
              
! calculate the sum of the weightings
              sum_weights = info_1%relative_weight + local_weight_2

! calculate a combined data value
              data_out(i_node) = &
     &          ( info_1%relative_weight * data_1(i_node) + &
     &            local_weight_2         * data_2(i_node) ) / &
     &          sum_weights

            enddo

          endif  ! got_data_2 block
          
        endif    ! got_data_1 block

      return
      end
!-----------------------------------------------------------------------
#endif             /* USE_NETCDF end of block */
!-----------------------------------------------------------------------
